<?php
/*
Plugin Name: Xspf_player
Plugin URI: http://www.boriel.com/?page_id=12
Description: Provides a quick a simple way to put MP3 soundtracks in your wordpress site. Music can be either local or remote (http). <br /> Uses free flash <a href="http://musicplayer.sourceforge.net">XSPF music player</a>.
Version: 1.1
Author: Jose Rodriguez (a.k.a. Boriel)
Author URI: http://www.boriel.com
*/

/*  Copyright 2004  Jose Rodriguez (a.k.a. Boriel)  (email: boriel@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// XSPF Player class 
// Defines a local xspf_ namespace


class xspf_player {
	// -----------------------------------------------------------------------------------------------------
	// -- CONFIGURATION --
	// You can change this make some hacks on this plugin (usually not needed)

	// Not for older versions of PHP 4.x???  */
	//const table_xspf = 'xspf_player'; // Name of the database table used for storing xspf tracks info
	//const table_xspf_categories = 'xspf_player_categories';
	//const table_xspf_cat_rel = 'xspf_player_tracks_categories';
	
	
	// -- CONFIGURATION END --
	// You should not edit anything beyond this line
	// -----------------------------------------------------------------------------------------------------
	

	// -----------------------------------------------------------------------------------------------------
	// Returns database xspf main table name
	// -----------------------------------------------------------------------------------------------------
	function table() {
		global $table_prefix;
		return $table_prefix . 'xspf_player';
	}
	
	// -----------------------------------------------------------------------------------------------------
	// Returns database categories table name
	// -----------------------------------------------------------------------------------------------------
	function table_categories() {
		global $table_prefix;
		return $table_prefix . 'xspf_player_categories';
	}
	
	// -----------------------------------------------------------------------------------------------------
	// Returns database categories <--> tracks table name
	// -----------------------------------------------------------------------------------------------------
	function table_cat_rel() {
		global $table_prefix;
		return $table_prefix . 'xspf_player_tracks_categories';
	}

	// -----------------------------------------------------------------------------------------------------
	// Constructor. 
	// This should be called when the plugin is installed.
	// It will create the wp_xspf_tracks table into the wp database
	// -----------------------------------------------------------------------------------------------------
	function xspf_player() {
		global $wpdb;
		global $table_prefix;
		
		// create table if not done already
		$query = 'CREATE TABLE IF NOT EXISTS `' . xspf_player::table() . "` (`id` int(11) NOT NULL auto_increment, " 
			. "`artist` tinytext, `title` tinytext, `url` varchar(128) UNIQUE NOT NULL, `imageurl` tinytext, `infourl` tinytext, PRIMARY KEY(`id`))";
		$wpdb->query($query);
		
		$query = 'CREATE TABLE IF NOT EXISTS `' . xspf_player::table_categories() . "`(`id` int(11) NOT NULL auto_increment, `name` varchar(64) UNIQUE NOT NULL, "
			. "`description` varchar(128) DEFAULT '', PRIMARY KEY(`id`))";
		$wpdb->query($query);
		
		$query = 'CREATE TABLE IF NOT EXISTS `' . xspf_player::table_cat_rel() . '` (`idtrack` int(11) NOT NULL, `idcat` int(11) NOT NULL, UNIQUE (idtrack, idcat), '
			. 'FOREIGN KEY (`idtrack`) REFERENCES `' . xspf_player::table() . '`(`id`) ON DELETE CASCADE ON UPDATE CASCADE, '
			. 'FOREIGN KEY (`idcat`) REFERENCES `' . xspf_player::table_categories() . '`(`id`) ON DELETE CASCADE ON UPDATE CASCADE)';
		$wpdb->query($query);
		
		add_action('admin_menu', array('xspf_player', 'add_pages'));
	}
	
	
	// -----------------------------------------------------------------------------------------------------
	// Shows a song data edition form body (used either for adding or updating song data)
	// Only form body (neither <form> nor <submit> tags.
	// -----------------------------------------------------------------------------------------------------
	function song_form_body($trackid = FALSE) {
		global $wpdb;
		
		if ($trackid === FALSE) {
			$artist = $title = $imageurl =  $url = $infourl = '';
		} else {
			$row = $wpdb->get_row('SELECT * FROM ' . xspf_player::table() . " WHERE id = $trackid");
			foreach ($row as $key => $value) {  // Expands SQL result to variables
				$$key = $value;
			}
		}
        ?>
	 <table width="100%" cellspacing="2" cellpadding="5" class="editform">
	  <tr>
	   <th>MP3 Track URL:</th>
	   <td>
		<input type="text" name="url" size="64" value="<?php echo $url; ?>" /> <br />
					Enter the absolute URL of the MP3 file. <strong>Required.</strong>
	   </td>
	  </tr>
	  <tr>
	   <th>Artist:</th>
	   <td>
			<input type="text" name="artist" value="<?php echo $artist; ?>" /> <br />
			Artist or Group name.
	   </td>
	  </tr>
	  <tr>
	   <th>Title:</th>
	   <td>
			<input type="text" name="title" value="<?php echo $title; ?>" /> <br />
			Track Title.
	   </td>
	  </tr>
	  <tr>
	   <th>Image URL:</th>
	   <td>
		   <input type="text" name="imageurl" size="64" value="<?php echo $imageurl; ?>" /> <br />
		   Little thumbnail image-like url (e.g. cd-cover). See XSPF player web.
	   </td>
	  </tr>
	  <tr>
	   <th>Artist URL:</th>
	   <td>
		   <input type="text" name="infourl" size="64" value="<?php echo $infourl; ?>" /> <br />
		   Artist or track info URL.
	   </td>
	  </tr> <?php
        if ($trackid != FALSE) {
            printf("<input type=\"hidden\" name=\"idtrack\" value=\"$trackid\" />\n");
        } ?>
	 </table>
	<?php
	}
	
	
	// -----------------------------------------------------------------------------------------------------
	// Shows song data table for edition. This function allows offset and limit for pagination.
	// -----------------------------------------------------------------------------------------------------
	function show_songs_table($from = '0', $limit = '10') {
		global $wpdb;
					
		$numrows = $wpdb->query("SELECT * FROM `" . xspf_player::table() . "` LIMIT $from, $limit");
		if ($numrows) {
		?>
		<fieldset id="tracks_div">
        <legend>Track List</legend>
		<table width="100%" cellpadding="3" cellspacing="3"> 
		 <tr>
		  <th scope="col">Title</th>
		  <th scope="col">Author</th>
		  <th scope="col">Image</th>
		  <th></th>
		  <th></th>
		 </tr>
		<?php 
			$results = $wpdb->get_results("SELECT * FROM `" . xspf_player::table() . "` LIMIT $from, $limit", ARRAY_A);
			foreach ($results as $row) { ?>
				<tr class='alternate'>
				 <td>
				  <a href="<?php echo $row['url']; ?>"><?php echo $row['title']; ?></a>
				 </td>
				 <td>
				 <?php if ($row['infourl'] != '') {
							printf('<a href="%s">%s</a>', $row['infourl'], $row['artist']);
					   } else {
							print $row['artist'];
					   }
				 ?>
				 </td>
				 <td>
				 <?php if ($row['imageurl'] != '') {
							printf('<img align="center" src="%s" width="20" height="20" />', $row['imageurl']);
					   }				 	
				 ?>
				 </td>
				 <form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
					 <input type="hidden" name="idtrack" value="<?php echo $row['id']; ?>">
				 <td><input type="submit" name="xspf_mgmt" value="<?php _e('Edit'); ?>"></td>
				 <td><input type="submit" name="xspf_mgmt" value="<?php _e('Delete'); ?>" 
					onclick="return confirm('You are about to delete this track info (you must remove the file manually)!\n  \'OK\' to delete, \'Cancel\' to stop.')">
				 </td>
				 </form>					
				</tr>
		<?php
			} // foreach
		?>
		</table>
		</fieldset>
		<?php
		}  // if ($numrows)
	} // private function show_songs_table
	

	// -----------------------------------------------------------------------------------------------------
	// Show categories table for management
	// -----------------------------------------------------------------------------------------------------
	function show_categories_table() {
		global $wpdb;
		
		$query = 'SELECT * FROM `' . xspf_player::table_categories() . '` ORDER BY `name` ASC';
		if ($wpdb->query($query)) { // There's at less 1 category
			?><div class="wrap"><?php
			$result = $wpdb->get_results($query, ARRAY_A);
			
		?><table width="100%" cellspacing="2" cellpadding="5" class="editform">
           <tr>
            <th>Id</th><th>Name</th><th>Description</th><th></th><th></th>
           </tr>
            <?php
			foreach ($result as $row) {?>
				<tr class="alternate">
				   <form action="" method="post">				
				   <th><?php echo $row['id']; ?></th>
				       <input type="hidden" name="idcat" value="<?php echo $row['id']; ?>" />
				   <td align="right"><input type="text" value="<?php echo $row['name']; ?>" name="cat_name" /></td>
				   <td align="right"><input type="text" size="64" value="<?php echo $row['description']; ?>" name="desc" /></td>
				   <td><input type="submit" value="<?php _e('Update'); ?>" name="xspf_setup" /></td>
				   <td><input type="submit" value="<?php _e('Delete'); ?>" name="xspf_setup" onclick="return confirm('You are about to delete this category.\n  \'OK\' to delete, \'Cancel\' to stop.')" /></td>
				   </form>				   
				</tr>
		<?php
			}
			print "</table>\n</div>\n";
		}
	}

	
	// -----------------------------------------------------------------------------------------------------
	// Add options into management menus
	// -----------------------------------------------------------------------------------------------------
	function add_pages() {
		add_management_page('XSPF Player Plugin Management', 'XSPF Player', 8, __FILE__, array('xspf_player', 'management'));
		add_options_page('XSPF Player Plugin Options', 'XSPF Player', 8, __FILE__, array('xspf_player', 'options'));
	}


	// -----------------------------------------------------------------------------------------------------
	// Shows plugin options
	// -----------------------------------------------------------------------------------------------------
	function options() {
		global $wpdb;
		
		if ($_POST['xspf_setup'] === 'sent') {  // Process pending SETUP information
			update_option('xspf_player_url', $wpdb->escape($_POST['player_url']));
			update_option('xspf_player_autostart', $wpdb->escape($_POST['player_autostart']));
			update_option('xspf_player_repeat', $wpdb->escape($_POST['player_repeat']));
			update_option('xspf_player_width', $wpdb->escape($_POST['width']));
			update_option('xspf_player_height', $wpdb->escape($_POST['height']));
		} elseif ($_POST['xspf_setup'] === 'add_category') { // Adds a new category
			$cat = $wpdb->escape(trim($_POST['new_cat']));
			if ($cat != '') { // Not empty string and...
		          if (!$wpdb->query('SELECT * FROM `' . xspf_player::table_categories() . "` WHERE `name` = '$cat'")) {  // Not already inserted
				        $wpdb->query('INSERT INTO `' . xspf_player::table_categories() . "`(`name`) VALUES ('$cat')");
                  }
			}
		} elseif ($_POST['xspf_setup'] === __('Delete')) { // Deletes a category
            // This should be done automatically by MySQL database (DELETE CASCADE) but depends on table types and versions...
            $wpdb->query('DELETE FROM `' . xspf_player::table_cat_rel() ."` WHERE idtrack = '" . $wpdb->escape($_POST['idcat']) . "'");
            
            // Delete category from categories table
            $wpdb->query('DELETE FROM `' . xspf_player::table_categories() ."` WHERE id = '" . $wpdb->escape($_POST['idcat']) . "'");
        } elseif ($_POST['xspf_setup'] === __('Update')) { // Updates a category values
            $cat = $wpdb->escape(trim($_POST['cat_name']));
            $result = $wpdb->get_var('SELECT `id` FROM `' . xspf_player::table_categories() . "` WHERE `name` = '$cat'");
            if (is_null($result) || ($result == $_POST['idcat'])) {
                $wpdb->query('UPDATE `' . xspf_player::table_categories() . "` SET `name` = '$cat', `description` = '"
                    . $wpdb->escape($_POST['desc']) . "' WHERE id = '" . $wpdb->escape($_POST['idcat']) . "'");
            }
        }
	?>	   
	   <div class="wrap">
		<h2>XSPF Player plugin Setup</h2>
		<form action="" method="post" name="xspf_prefs">
		 <input type="hidden" name="xspf_setup" value="sent" />	   
		 <table width="100%" cellspacing="2" cellpadding="5" class="editform"> 
		  <tr>
		   <th>XSPF Flash Player URL:</th>
		   <td>
			<input type="text" name="player_url" value="<?php $player_url = get_option('xspf_player_url');
														if ($player_url == '') {
															$player_url = 'http://musicplayer.sourceforge.net/xspf_player_slim.swf';
														}
														echo $player_url;
												 ?>" size="64" /> <br />
			Enter the URL of your xspf player. Please, use your webserver, not this default URL, if possible. You can download it from <a href="http://musicplayer.sourceforge.net">http://musicplayer.sourceforge.net</a>
		   </td>
		  </tr>
		  <tr>
		   <th>Autostart:</th>
		   <td>
			<input type="checkbox" <?php if (get_option('xspf_player_autostart') == 1) {
												echo 'checked';
										  }?> value="1" name="player_autostart" /> If checked, tracks will start playing automatically.
		   </td>
		  </tr>
		  <tr>
		   <th>Repeat:</th>
		   <td>
			<input type="checkbox" <?php if (get_option('xspf_player_repeat') == 1) {
												echo 'checked';
										  }?> value="1" name="player_repeat" /> Check to repeat the playlist after the end of the last song.
		   </td>
		  </tr>
		  <tr>
		   <td colspan="2">
		    <table align="center">
			 <tr>
		   	<th>Width:</th>
		   	<td><input type="text" value="<?php $width = get_option('xspf_player_width');
		   										if ('' == $width) {
													$width = '176';  // Default width
												}
												echo $width;
		   								 ?>" name="width" /></td>
			<th>Height:</th>
			<td>
		    <input type="text" value="<?php $height = get_option('xspf_player_height');
		   										if ('' == $height) {
													$height = '15';  // Default height
												}
												echo $height;
		   								 ?>" name="height" /></td>		   
		 
			 </tr>
		    </table>
		  </td>
		  </tr>
		 </table>
		 <p class="submit">
		  <input type="submit" name="action" class="submit" value="<?php _e('Update');?> &raquo;" />
		 </p>
		</form>
		<h2>Categories</h2>
		You can define categories and use them to select a subgroup of tracks.<br />
		Deleting a category does not delete a track. <br />
		A single track can be tagged under multiple categories. <br />
		<?php xspf_player::show_categories_table(); ?>
		<form action="" method="post">
            <table>
              <tr>
               <td><input type="text" name="new_cat" size="30" /></td>
               <td class="submit">
			    <input type="submit" class="submit" name="action" value="<?php _e('Add Category'); ?>&raquo;" />
			   </td>
              </tr>
			 <input type="hidden" name="xspf_setup" value="add_category" />
			</table>
		</form>
	   </div>
<?php
	} /* public function options() */
	
	// -----------------------------------------------------------------------------------------------------
	// Tracks management page
	// -----------------------------------------------------------------------------------------------------
	function management() {
		global $wpdb;
		
	    // -----------------------------------------------------------------------------------------------------
	    // Show a Box for categories for a song
	    // -----------------------------------------------------------------------------------------------------
		function show_categories_mgmt($idtrack = FALSE) {
            global $wpdb;
            
            ?>
            <fieldset id="track_categories">
             <legend>Track Categories</legend>
             <?php
           		$query = 'SELECT * FROM `' . xspf_player::table_categories() . '` ORDER BY `name` ASC';
                if (($result = $wpdb->get_results($query, ARRAY_A))) { // There's at less 1 category
                    $query = 'SELECT a.`id` FROM `' . xspf_player::table_categories() . '` a INNER JOIN `'
                        . xspf_player::table_cat_rel() . "` b ON (a.`id` = b.`idcat`) WHERE b.`idtrack` = '$idtrack'";

                    if (($cat_rels = $wpdb->get_col($query)) == NULL) {
                        $cat_rels = array();
                    }

                    foreach ($result as $row) { ?>
                        <label for="category-<?php echo $row['id'];?>" class="selectit">
                         <input value="1" type="checkbox" name="post_category[<?php echo $row['id']; ?>]" <?php
                            if (in_array($row['id'], $cat_rels)) {
                                    echo 'checked="checked"';
                            } ?> /> <?php echo $row['name']; ?>
                        </label><?php
                    }
                }
             ?>
            </fieldset>
            <?php
		}
		
	    // -----------------------------------------------------------------------------------------------------
	    // Tracks management page
        // -----------------------------------------------------------------------------------------------------
		function show_management($legend, $action, $button_label, $idtrack = FALSE) {
			xspf_player::show_songs_table();?>
			<form action="" method="post" name="xspf_mgmt">
             <table width="100%">
              <tr><td width="90%">
			 <fieldset id="track_form">
			  <legend><?php echo $legend; ?></legend>
				<input type="hidden" name="xspf_mgmt" value="<?php echo $action; ?>" />
			 <?php xspf_player::song_form_body($idtrack); ?>
			 <p class="submit">
			  <input type="submit" name="action" value="<?php echo $button_label; ?> &raquo;" />
			 </p>
			 </fieldset></td>
			 <td>
             <?php show_categories_mgmt($idtrack); ?>
             </td>
             </tr>
             </table>
			</form>
			<?php
		}
		?>
	   <div class="wrap">
		<h2>XSPF Player Plugin Management</h2>
	<?php
		switch ($_POST['xspf_mgmt']) {
			case __('Delete'):   // deleting a new track
				$wpdb->query('DELETE FROM `' . xspf_player::table_cat_rel() . '` WHERE `idtrack` = ' . $_POST['idtrack']);
				$wpdb->query('DELETE FROM `' . xspf_player::table() . '` WHERE `id` = ' . $_POST['idtrack']);
				show_management(__('Add New Track'), 'add', __('Add New Track'));				
				break;	
				
			case __('Edit'): // Editing old values
				show_management(__('Update Track'), 'update', __('Update Track Info'), $_POST['idtrack']);	
				break;
				
			case 'update': // Update Track Info
                $id = $wpdb->escape(trim($_POST['idtrack']));
                $wpdb->query('DELETE FROM `' . xspf_player::table_cat_rel() . "` WHERE `idtrack` = '$id'"); // Delete al relations
                if (is_array($_POST['post_category'])) {  //post_category is defined an is an array
                    foreach ($_POST['post_category'] as $idcat => $valor) {
                        if ($valor == 1) { // Category set?
                            $wpdb->query('INSERT INTO `' . xspf_player::table_cat_rel() . "`(`idtrack`, `idcat`) VALUES('$id', '$idcat')");
                        }
                    }
                }
                
				$url = trim($wpdb->escape(trim($_POST['url'])));
				if ($url === '') {  // Empty tracks url not allowed
					break;
				}
				$imageurl = $wpdb->escape(trim($_POST['imageurl']));
				$infourl = $wpdb->escape(trim($_POST['infourl']));
				$artist = $wpdb->escape($_POST['artist']);
				$title = $wpdb->escape($_POST['title']);
				$wpdb->query('UPDATE `' . xspf_player::table() . "` SET `artist` = '$artist', `title` = '$title', `imageurl` = '$imageurl', `url` = '$url', `infourl` = '$infourl'"
                    . " WHERE `id` = '$id'");
				show_management(__('Add New Track'), 'add', __('Add New Track'));				
				break;

			case 'add':   // adding a new track
				$url = trim($wpdb->escape(trim($_POST['url'])));
				if ($url === '') {  // Empty tracks url not allowed
					break;
				}
			
				$imageurl = $wpdb->escape(trim($_POST['imageurl']));
				$infourl = $wpdb->escape(trim($_POST['infourl']));
				$artist = $wpdb->escape($_POST['artist']);
				$title = $wpdb->escape($_POST['title']);

				$wpdb->query('INSERT INTO `' . xspf_player::table() . "`(`artist`, `title`, `imageurl`, `url`, `infourl`) VALUES ('$artist', '$title', '$imageurl', '$url', '$infourl')");
				$id = $wpdb->get_var('SELECT `id` FROM `' . xspf_player::table() . "` WHERE `url`= '$url'");
                $wpdb->query('DELETE FROM `' . xspf_player::table_cat_rel() . "` WHERE `idtrack` = '$id'"); // Delete al relations (should not be needed)
                if (is_array($_POST['post_category'])) {  //post_category is defined an is an array
                    foreach ($_POST['post_category'] as $idcat => $valor) {
                        if ($valor == 1) { // Category set?
                            $wpdb->query('INSERT INTO `' . xspf_player::table_cat_rel() . "`(`idtrack`, `idcat`) VALUES('$id', '$idcat')");
                        }
                    }
                }				
				
			default:
				show_management(__('Add New Track'), 'add', __('Add New Track'));
		}
	?></div><?php
	} /* public function management() */
	
	
	// -----------------------------------------------------------------------------------------------------
	// This function put the player on your web page selecting songs by category (Default: ALL)
	// -----------------------------------------------------------------------------------------------------
	function start($category = FALSE) {
        global $wpdb;
        
        if ($category != FALSE) {
            $category = $wpdb->get_var('SELECT `id` FROM `' . xspf_player::table_categories() . "` WHERE `name` = '$category'");
            if (is_null($category)) {  // If the category does not exists, "ALL" is selected (using FALSE)
                $category = FALSE;
            }
        }
		
		if (($w = get_option('xspf_player_width')) == '') {
			$w = 176;
		}
		
		if (($h = get_option('xspf_player_height')) == '') {
			$h = 176;
		}
        
		$protocol = (isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] === ''))? 'https': 'http';
		$url = $protocol .'://' . $_SERVER['SERVER_NAME'] . dirname($_SERVER['PHP_SELF']);
		if (substr($url, -1) != '/') {
			$url .= '/';
		}
		$url .= 'wp-content/plugins/xspf_player/playlist.php';
		?>
		<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000"
			codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0"
			width="<?php echo $w; ?>" height="<?php echo $h; ?>" >
		 <param name="allowScriptAccess" value="sameDomain"/>
		 <param name="movie" value="<?php echo get_option('xspf_player_url'); ?>"/>
		 <param name="quality" value="high"/>
		 <param name="bgcolor" value="#E6E6E6"/>
		 <embed src="<?php echo get_option('xspf_player_url'); ?>?<?php 
		 										if (get_option('xspf_player_autostart') == 1) {
													echo 'autoplay=true&';
												}
												
												if (get_option('xspf_player_repeat') == 1) {
													echo 'repeat_playlist=true&';
												}			   ?>playlist_url=<?php echo $url; 
																if ($category != FALSE) {
																	echo "?cat=$category";
																}?>" 
		 quality="high" bgcolor="#E6E6E6" name="xspf_player_slim" allowscriptaccess="sameDomain"
		 type="application/x-shockwave-flash"
		 pluginspage="http://www.macromedia.com/go/getflashplayer"
		 align="center" height="<?php echo $h;?>" width="<?php echo $w;?>"> </embed>
		</object> <?php
	}  /* public function start() */
}

// main() 
$xspf = new xspf_player();  // Initialize the plugin (database, etc...)
                            // It's safe to do this more than once.

?>
